/*
 * Copyright (c) 2002-2004 David Keiichi Watanabe
 * davew@xlife.org
 *
 * Modified by (c) 2004-2005 heavy_baby
 * heavy_baby@users.sourceforge.jp
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jp.sourceforge.cabos;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;

import com.limegroup.gnutella.RouterService;
import com.limegroup.gnutella.Uploader;
import com.limegroup.gnutella.downloader.ManagedDownloader;
import com.limegroup.gnutella.settings.ApplicationSettings;
import com.limegroup.gnutella.util.CommonUtils;
import com.limegroup.gnutella.util.FileUtils;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.LinkedList;
import com.sun.java.util.collections.List;

public class AqMain {
	/* Instances */

	private static InputStreamReader reader = null;

	private static OutputStreamWriter writer = null;

	private static Socket socket = null;

	private static boolean isShuttingdown = false;

	/* Main */

	public static void main(String[] args) {
		/* open streams */

		try {
			socket = new Socket("localhost", 1024);
			reader = new InputStreamReader(socket.getInputStream(), "UTF-8");
			writer = new OutputStreamWriter(socket.getOutputStream(), "UTF-8");
		} catch (Exception e) {
			System.exit(0);
		}

		AqEvent.signalEvent(AqEvent.kLWEventCoreConnected);

		/* make writable app support directories */

		if (CommonUtils.getUserHomeDir().exists())
			FileUtils.setWriteable(CommonUtils.getUserHomeDir());
		else
			CommonUtils.getUserHomeDir().mkdirs();
		if (CommonUtils.getUserSettingsDir().exists())
			FileUtils.setWriteable(CommonUtils.getUserSettingsDir());
		else
			CommonUtils.getUserSettingsDir().mkdirs();

		/* read loop */

		AqEvent.signalEvent(AqEvent.kLWEventCoreInitialized);

		readLoop();
		System.exit(0);
	}

	private static void readLoop() {

		List args = new LinkedList();
		StringBuffer fragment = new StringBuffer();

		try {
			while (!isShuttingdown) {
				char c = (char) reader.read();

				if (c == '|' || c == '\n' || c == '\r') {
					args.add(fragment.toString());
					fragment.setLength(0);

					if (c == '\n' || c == '\r') {
						AqDispatcher.dispatchCommand(args);
						args.clear();
					}

				} else {
					fragment.append(c);
				}
			}
		} catch (IOException e) {
		}

	}

	protected static void start() {
		/* start LW */

		RouterService router = new RouterService(new AqEventHandler());
		router.start();

		/* schedule threads */

		RouterService.schedule(new ConnectionUpdate(), 60 * 1000, 60 * 1000);
		RouterService.schedule(new TransferUpdate(), 1 * 1000, 1 * 1000);
	}

	protected static void writeEvent(String event) {
		synchronized (writer) {
			try {
				writer.write(event);
				writer.flush();
			} catch (IOException e) {
			}
		}
	}

	protected static void shutdown() {
		synchronized (writer) {
			try {
				reader.close();
				writer.close();
				socket.close();
			} catch (IOException e) {
			}
			isShuttingdown = true;
		}
	}
}

/* ConnectionUpdate */

class ConnectionUpdate implements Runnable {
	public void run() {
		AqEvent.signalEvent(AqEvent.kLWEventConnectionsUpdated);
	}
}

/* TransferUpdate */

class TransferUpdate implements Runnable {

	public void run() {
		/* calculate timer */

		int totalUptime = ApplicationSettings.TOTAL_UPTIME.getValue() + 1;
		ApplicationSettings.TOTAL_UPTIME.setValue(totalUptime);
		ApplicationSettings.AVERAGE_UPTIME.setValue(totalUptime
				/ ApplicationSettings.SESSIONS.getValue());

		/* Downloads */

		for (Iterator i = RouterService.getDownloadManager().getDownloads()
				.iterator(); i.hasNext(); publishUpdate((ManagedDownloader) i
				.next()))
			;
		AqEvent.signalEvent(AqEvent.kLWEventDownloadsUpdated);

		/* Uploads */

		for (Iterator i = RouterService.getUploadManager().getUploads()
				.iterator(); i.hasNext(); publishUpdate((Uploader) i.next()))
			;
		AqEvent.signalEvent(AqEvent.kLWEventUploadsUpdated);
	}

	private void publishUpdate(ManagedDownloader d) {
		if (d != null)
			AqEvent.signalEvent(AqEvent.kLWEventUpdateDownloadStats, d);
	}

	private void publishUpdate(Uploader u) {
		if (u != null)
			AqEvent.signalEvent(AqEvent.kLWEventUpdateUploadStats, u);
	}

}
